@@ -714,58 +714,6 @@ edge_point_side(const GEOGRAPHIC_EDGE *e, const GEOGRAPHIC_POINT *p)
 		return 1;
 }
 
-/**
-* Returns the angle in radians at point B of the triangle formed by A-B-C
-*/
-static double
-sphere_angle(const GEOGRAPHIC_POINT *a, const GEOGRAPHIC_POINT *b,  const GEOGRAPHIC_POINT *c)
-{
-	POINT3D normal1, normal2;
-	robust_cross_product(b, a, &normal1);
-	robust_cross_product(b, c, &normal2);
-	normalize(&normal1);
-	normalize(&normal2);
-	return sphere_distance_cartesian(&normal1, &normal2);
-}
-
-/**
-* Computes the spherical area of a triangle. If C is to the left of A/B,
-* the area is negative. If C is to the right of A/B, the area is positive.
-*
-* @param a The first triangle vertex.
-* @param b The second triangle vertex.
-* @param c The last triangle vertex.
-* @return the signed area in radians.
-*/
-static double
-sphere_signed_area(const GEOGRAPHIC_POINT *a, const GEOGRAPHIC_POINT *b, const GEOGRAPHIC_POINT *c)
-{
-	double angle_a, angle_b, angle_c;
-	double area_radians = 0.0;
-	int side;
-	GEOGRAPHIC_EDGE e;
-
-	angle_a = sphere_angle(b,a,c);
-	angle_b = sphere_angle(a,b,c);
-	angle_c = sphere_angle(b,c,a);
-
-	area_radians = angle_a + angle_b + angle_c - M_PI;
-
-	/* What's the direction of the B/C edge? */
-	e.start = *a;
-	e.end = *b;
-	side = edge_point_side(&e, c);
-
-	/* Co-linear points implies no area */
-	if ( side == 0 )
-		return 0.0;
-
-	/* Add the sign to the area */
-	return side * area_radians;
-}
-
-
-
 /**
 * Returns true if the point p is on the great circle plane.
 * Forms the scalar triple product of A,B,p and if the volume of the
@@ -947,17 +895,21 @@ int edge_contains_coplanar_point(const GEOGRAPHIC_EDGE *e, const GEOGRAPHIC_POIN
 */
 double sphere_distance(const GEOGRAPHIC_POINT *s, const GEOGRAPHIC_POINT *e)
 {
-	double d_lon = e->lon - s->lon;
-	double cos_d_lon = cos(d_lon);
-	double cos_lat_e = cos(e->lat);
-	double sin_lat_e = sin(e->lat);
-	double cos_lat_s = cos(s->lat);
-	double sin_lat_s = sin(s->lat);
+	double d_lon, cos_d_lon, cos_lat_e, sin_lat_e, cos_lat_s, sin_lat_s;
+	double a1, a2, a, b;
 
-	double a1 = POW2(cos_lat_e * sin(d_lon));
-	double a2 = POW2(cos_lat_s * sin_lat_e - sin_lat_s * cos_lat_e * cos_d_lon);
-	double a = sqrt(a1 + a2);
-	double b = sin_lat_s * sin_lat_e + cos_lat_s * cos_lat_e * cos_d_lon;
+	if (FP_EQUALS(s->lat, e->lat) && FP_EQUALS(s->lon, e->lon)) return 0.0;
+	d_lon = e->lon - s->lon;
+	cos_d_lon = cos(d_lon);
+	cos_lat_e = cos(e->lat);
+	sin_lat_e = sin(e->lat);
+	cos_lat_s = cos(s->lat);
+	sin_lat_s = sin(s->lat);
+
+	a1 = POW2(cos_lat_e * sin(d_lon));
+	a2 = POW2(cos_lat_s * sin_lat_e - sin_lat_s * cos_lat_e * cos_d_lon);
+	a = sqrt(a1 + a2);
+	b = sin_lat_s * sin_lat_e + cos_lat_s * cos_lat_e * cos_d_lon;
 	return atan2(a, b);
 }
 
@@ -1801,39 +1753,6 @@ lwgeom_segmentize_sphere(const LWGEOM *lwg_in, double max_seg_length)
 }
 
 
-/**
-* Returns the area of the ring (ring must be closed) in square radians (surface of
-* the sphere is 4*PI).
-*/
-double
-ptarray_area_sphere(const POINTARRAY *pa)
-{
-	uint32_t i;
-	const POINT2D *p;
-	GEOGRAPHIC_POINT a, b, c;
-	double area = 0.0;
-
-	/* Return zero on nonsensical inputs */
-	if ( ! pa || pa->npoints < 4 )
-		return 0.0;
-
-	p = getPoint2d_cp(pa, 0);
-	geographic_point_init(p->x, p->y, &a);
-	p = getPoint2d_cp(pa, 1);
-	geographic_point_init(p->x, p->y, &b);
-
-	for ( i = 2; i < pa->npoints-1; i++ )
-	{
-		p = getPoint2d_cp(pa, i);
-		geographic_point_init(p->x, p->y, &c);
-		area += sphere_signed_area(&a, &b, &c);
-		b = c;
-	}
-
-	return fabs(area);
-}
-
-
 static double ptarray_distance_spheroid(const POINTARRAY *pa1, const POINTARRAY *pa2, const SPHEROID *s, double tolerance, int check_intersection)
 {
 	GEOGRAPHIC_EDGE e1, e2;
@@ -1913,7 +1832,7 @@ static double ptarray_distance_spheroid(const POINTARRAY *pa1, const POINTARRAY
 				nearest2 = g2;
 			}
 			/* We've gotten closer than the tolerance... */
-			if ( d < tolerance )
+			if ( d <= tolerance )
 			{
 				/* Working on a sphere? The answer is correct, return */
 				if ( use_sphere )
@@ -1921,7 +1840,7 @@ static double ptarray_distance_spheroid(const POINTARRAY *pa1, const POINTARRAY
 					return d;
 				}
 				/* Far enough past the tolerance that the spheroid calculation won't change things */
-				else if ( d < tolerance * 0.95 )
+				else if ( d <= tolerance * 0.95 )
 				{
 					return d;
 				}
@@ -1930,7 +1849,7 @@ static double ptarray_distance_spheroid(const POINTARRAY *pa1, const POINTARRAY
 				{
 					d = spheroid_distance(&g1, &nearest2, s);
 					/* Yes, closer than tolerance, return! */
-					if ( d < tolerance )
+					if ( d <= tolerance )
 						return d;
 				}
 			}
@@ -1991,7 +1910,7 @@ static double ptarray_distance_spheroid(const POINTARRAY *pa1, const POINTARRAY
 				nearest1 = g1;
 				nearest2 = g2;
 			}
-			if ( d < tolerance )
+			if ( d <= tolerance )
 			{
 				if ( use_sphere )
 				{
@@ -2000,7 +1919,7 @@ static double ptarray_distance_spheroid(const POINTARRAY *pa1, const POINTARRAY
 				else
 				{
 					d = spheroid_distance(&nearest1, &nearest2, s);
-					if ( d < tolerance )
+					if ( d <= tolerance )
 						return d;
 				}
 			}
@@ -2025,67 +1944,14 @@ static double ptarray_distance_spheroid(const POINTARRAY *pa1, const POINTARRAY
 
 
 /**
-* Calculate the area of an LWGEOM. Anything except POLYGON, MULTIPOLYGON
-* and GEOMETRYCOLLECTION return zero immediately. Multi's recurse, polygons
-* calculate external ring area and subtract internal ring area. A GBOX is
-* required to calculate an outside point.
+* Delegate to the spheroid function with a spherically
+* parameterized spheroid.
 */
 double lwgeom_area_sphere(const LWGEOM *lwgeom, const SPHEROID *spheroid)
 {
-	int type;
-	double radius2 = spheroid->radius * spheroid->radius;
-
-	assert(lwgeom);
-
-	/* No area in nothing */
-	if ( lwgeom_is_empty(lwgeom) )
-		return 0.0;
-
-	/* Read the geometry type number */
-	type = lwgeom->type;
-
-	/* Anything but polygons and collections returns zero */
-	if ( ! ( type == POLYGONTYPE || type == MULTIPOLYGONTYPE || type == COLLECTIONTYPE ) )
-		return 0.0;
-
-	/* Actually calculate area */
-	if ( type == POLYGONTYPE )
-	{
-		LWPOLY *poly = (LWPOLY*)lwgeom;
-		uint32_t i;
-		double area = 0.0;
-
-		/* Just in case there's no rings */
-		if ( poly->nrings < 1 )
-			return 0.0;
-
-		/* First, the area of the outer ring */
-		area += radius2 * ptarray_area_sphere(poly->rings[0]);
-
-		/* Subtract areas of inner rings */
-		for ( i = 1; i < poly->nrings; i++ )
-		{
-			area -= radius2 * ptarray_area_sphere(poly->rings[i]);
-		}
-		return area;
-	}
-
-	/* Recurse into sub-geometries to get area */
-	if ( type == MULTIPOLYGONTYPE || type == COLLECTIONTYPE )
-	{
-		LWCOLLECTION *col = (LWCOLLECTION*)lwgeom;
-		uint32_t i;
-		double area = 0.0;
-
-		for ( i = 0; i < col->ngeoms; i++ )
-		{
-			area += lwgeom_area_sphere(col->geoms[i], spheroid);
-		}
-		return area;
-	}
-
-	/* Shouldn't get here. */
-	return 0.0;
+	SPHEROID s;
+	spheroid_init(&s, WGS84_RADIUS, WGS84_RADIUS);
+	return lwgeom_area_spheroid(lwgeom, &s);
 }
 
 
@@ -2214,13 +2080,13 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 	type2 = lwgeom2->type;
 
 	/* Make sure we have boxes */
-	if ( lwgeom1->bbox )
+	if ( FLAGS_GET_GEODETIC(lwgeom1->flags) && lwgeom1->bbox )
 		gbox1 = *(lwgeom1->bbox);
 	else
 		lwgeom_calculate_gbox_geodetic(lwgeom1, &gbox1);
 
 	/* Make sure we have boxes */
-	if ( lwgeom2->bbox )
+	if ( FLAGS_GET_GEODETIC(lwgeom2->flags) && lwgeom2->bbox )
 		gbox2 = *(lwgeom2->bbox);
 	else
 		lwgeom_calculate_gbox_geodetic(lwgeom2, &gbox2);
@@ -2282,7 +2148,7 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 			double ring_distance = ptarray_distance_spheroid(lwpoly->rings[i], lwpt->point, spheroid, tolerance, check_intersection);
 			if ( ring_distance < distance )
 				distance = ring_distance;
-			if ( distance < tolerance )
+			if ( distance <= tolerance )
 				return distance;
 		}
 		return distance;
@@ -2325,7 +2191,7 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 			LWDEBUGF(4, "ring[%d] ring_distance = %.8g", i, ring_distance);
 			if ( ring_distance < distance )
 				distance = ring_distance;
-			if ( distance < tolerance )
+			if ( distance <= tolerance )
 				return distance;
 		}
 		LWDEBUGF(4, "all rings checked, returning distance = %.8g", distance);
@@ -2365,7 +2231,7 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 					check_intersection);
 				if (ring_distance < distance)
 					distance = ring_distance;
-				if (distance < tolerance) return distance;
+				if (distance <= tolerance) return distance;
 			}
 		}
 		return distance;
@@ -2384,7 +2250,7 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 			    col->geoms[i], lwgeom2, spheroid, tolerance);
 			if ( geom_distance < distance )
 				distance = geom_distance;
-			if ( distance < tolerance )
+			if ( distance <= tolerance )
 				return distance;
 		}
 		return distance;
@@ -2402,7 +2268,7 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 			double geom_distance = lwgeom_distance_spheroid(lwgeom1, col->geoms[i], spheroid, tolerance);
 			if ( geom_distance < distance )
 				distance = geom_distance;
-			if ( distance < tolerance )
+			if ( distance <= tolerance )
 				return distance;
 		}
 		return distance;
@@ -3338,22 +3204,22 @@ ptarray_nudge_geodetic(POINTARRAY *pa)
 	for(i = 0; i < pa->npoints; i++ )
 	{
 		getPoint4d_p(pa, i, &p);
-		if ( p.x < -180.0 && (-180.0 - p.x < tolerance) )
+		if ( p.x < -180.0 && (-180.0 - p.x <= tolerance) )
 		{
 			p.x = -180.0;
 			altered = LW_TRUE;
 		}
-		if ( p.x > 180.0 && (p.x - 180.0 < tolerance) )
+		if ( p.x > 180.0 && (p.x - 180.0 <= tolerance) )
 		{
 			p.x = 180.0;
 			altered = LW_TRUE;
 		}
-		if ( p.y < -90.0 && (-90.0 - p.y < tolerance) )
+		if ( p.y < -90.0 && (-90.0 - p.y <= tolerance) )
 		{
 			p.y = -90.0;
 			altered = LW_TRUE;
 		}
-		if ( p.y > 90.0 && (p.y - 90.0 < tolerance) )
+		if ( p.y > 90.0 && (p.y - 90.0 <= tolerance) )
 		{
 			p.y = 90.0;
 			altered = LW_TRUE;
