@@ -28,13 +28,14 @@
 #include <string.h>	/* strlen */
 #include <assert.h>
 
-static char *asgeojson_point(const LWPOINT *point, char *srs, GBOX *bbox, int precision);
-static char *asgeojson_line(const LWLINE *line, char *srs, GBOX *bbox, int precision);
-static char *asgeojson_poly(const LWPOLY *poly, char *srs, GBOX *bbox, int precision);
-static char * asgeojson_multipoint(const LWMPOINT *mpoint, char *srs, GBOX *bbox, int precision);
-static char * asgeojson_multiline(const LWMLINE *mline, char *srs, GBOX *bbox, int precision);
-static char * asgeojson_multipolygon(const LWMPOLY *mpoly, char *srs, GBOX *bbox, int precision);
-static char * asgeojson_collection(const LWCOLLECTION *col, char *srs, GBOX *bbox, int precision);
+static lwvarlena_t *asgeojson_point(const LWPOINT *point, const char *srs, GBOX *bbox, int precision);
+static lwvarlena_t *asgeojson_line(const LWLINE *line, const char *srs, GBOX *bbox, int precision);
+static lwvarlena_t *asgeojson_triangle(const LWTRIANGLE *tri, const char *srs, GBOX *bbox, int precision);
+static lwvarlena_t *asgeojson_poly(const LWPOLY *poly, const char *srs, GBOX *bbox, int precision);
+static lwvarlena_t *asgeojson_multipoint(const LWMPOINT *mpoint, const char *srs, GBOX *bbox, int precision);
+static lwvarlena_t *asgeojson_multiline(const LWMLINE *mline, const char *srs, GBOX *bbox, int precision);
+static lwvarlena_t *asgeojson_multipolygon(const LWMPOLY *mpoly, const char *srs, GBOX *bbox, int precision);
+static lwvarlena_t *asgeojson_collection(const LWCOLLECTION *col, const char *srs, GBOX *bbox, int precision);
 static size_t asgeojson_geom_size(const LWGEOM *geom, GBOX *bbox, int precision);
 static size_t asgeojson_geom_buf(const LWGEOM *geom, char *output, GBOX *bbox, int precision);
 
@@ -44,14 +45,12 @@ static size_t pointArray_geojson_size(POINTARRAY *pa, int precision);
 /**
  * Takes a GEOMETRY and returns a GeoJson representation
  */
-char *
-lwgeom_to_geojson(const LWGEOM *geom, char *srs, int precision, int has_bbox)
+lwvarlena_t *
+lwgeom_to_geojson(const LWGEOM *geom, const char *srs, int precision, int has_bbox)
 {
 	int type = geom->type;
 	GBOX *bbox = NULL;
-	GBOX tmp;
-
-	if ( precision > OUT_MAX_DOUBLE_PRECISION ) precision = OUT_MAX_DOUBLE_PRECISION;
+	GBOX tmp = {0};
 
 	if (has_bbox)
 	{
@@ -75,6 +74,9 @@ lwgeom_to_geojson(const LWGEOM *geom, char *srs, int precision, int has_bbox)
 		return asgeojson_multiline((LWMLINE*)geom, srs, bbox, precision);
 	case MULTIPOLYGONTYPE:
 		return asgeojson_multipolygon((LWMPOLY*)geom, srs, bbox, precision);
+	case TRIANGLETYPE:
+		return asgeojson_triangle((LWTRIANGLE *)geom, srs, bbox, precision);
+	case TINTYPE:
 	case COLLECTIONTYPE:
 		return asgeojson_collection((LWCOLLECTION*)geom, srs, bbox, precision);
 	default:
@@ -92,7 +94,7 @@ lwgeom_to_geojson(const LWGEOM *geom, char *srs, int precision, int has_bbox)
  * Handle SRS
  */
 static size_t
-asgeojson_srs_size(char *srs)
+asgeojson_srs_size(const char *srs)
 {
 	int size;
 
@@ -104,7 +106,7 @@ asgeojson_srs_size(char *srs)
 }
 
 static size_t
-asgeojson_srs_buf(char *output, char *srs)
+asgeojson_srs_buf(char *output, const char *srs)
 {
 	char *ptr = output;
 
@@ -127,12 +129,12 @@ asgeojson_bbox_size(int hasz, int precision)
 	if (!hasz)
 	{
 		size = sizeof("\"bbox\":[,,,],");
-		size +=	2 * 2 * (OUT_MAX_DIGS_DOUBLE + precision);
+		size += 2 * 2 * (OUT_MAX_BYTES_DOUBLE + precision);
 	}
 	else
 	{
 		size = sizeof("\"bbox\":[,,,,,],");
-		size +=	2 * 3 * (OUT_MAX_DIGS_DOUBLE + precision);
+		size += 2 * 3 * (OUT_MAX_BYTES_DOUBLE + precision);
 	}
 
 	return size;
@@ -162,7 +164,7 @@ asgeojson_bbox_buf(char *output, GBOX *bbox, int hasz, int precision)
  */
 
 static size_t
-asgeojson_point_size(const LWPOINT *point, char *srs, GBOX *bbox, int precision)
+asgeojson_point_size(const LWPOINT *point, const char *srs, GBOX *bbox, int precision)
 {
 	int size;
 
@@ -180,7 +182,7 @@ asgeojson_point_size(const LWPOINT *point, char *srs, GBOX *bbox, int precision)
 }
 
 static size_t
-asgeojson_point_buf(const LWPOINT *point, char *srs, char *output, GBOX *bbox, int precision)
+asgeojson_point_buf(const LWPOINT *point, const char *srs, char *output, GBOX *bbox, int precision)
 {
 	char *ptr = output;
 
@@ -197,26 +199,69 @@ asgeojson_point_buf(const LWPOINT *point, char *srs, char *output, GBOX *bbox, i
 	return (ptr-output);
 }
 
-static char *
-asgeojson_point(const LWPOINT *point, char *srs, GBOX *bbox, int precision)
+static lwvarlena_t *
+asgeojson_point(const LWPOINT *point, const char *srs, GBOX *bbox, int precision)
 {
-	char *output;
-	int size;
-
-	size = asgeojson_point_size(point, srs, bbox, precision);
-	output = lwalloc(size);
-	asgeojson_point_buf(point, srs, output, bbox, precision);
+	uint32_t size = asgeojson_point_size(point, srs, bbox, precision);
+	lwvarlena_t *output = (lwvarlena_t *)lwalloc(size + LWVARHDRSZ);
+	size = asgeojson_point_buf(point, srs, output->data, bbox, precision);
+	LWSIZE_SET(output->size, size + LWVARHDRSZ);
 	return output;
 }
 
+/**
+ * Triangle Geometry
+ */
 
+static size_t
+asgeojson_triangle_size(const LWTRIANGLE *tri, const char *srs, GBOX *bbox, int precision)
+{
+	int size;
+
+	size = sizeof("{'type':'Polygon',");
+	if (srs)
+		size += asgeojson_srs_size(srs);
+	if (bbox)
+		size += asgeojson_bbox_size(FLAGS_GET_Z(tri->flags), precision);
+	size += sizeof("'coordinates':[[]]}");
+	size += pointArray_geojson_size(tri->points, precision);
+
+	return size;
+}
+
+static size_t
+asgeojson_triangle_buf(const LWTRIANGLE *tri, const char *srs, char *output, GBOX *bbox, int precision)
+{
+	char *ptr = output;
+
+	ptr += sprintf(ptr, "{\"type\":\"Polygon\",");
+	if (srs)
+		ptr += asgeojson_srs_buf(ptr, srs);
+	if (bbox)
+		ptr += asgeojson_bbox_buf(ptr, bbox, FLAGS_GET_Z(tri->flags), precision);
+	ptr += sprintf(ptr, "\"coordinates\":[[");
+	ptr += pointArray_to_geojson(tri->points, ptr, precision);
+	ptr += sprintf(ptr, "]]}");
+
+	return (ptr - output);
+}
+
+static lwvarlena_t *
+asgeojson_triangle(const LWTRIANGLE *tri, const char *srs, GBOX *bbox, int precision)
+{
+	uint32_t size = asgeojson_triangle_size(tri, srs, bbox, precision);
+	lwvarlena_t *output = (lwvarlena_t *)lwalloc(size + LWVARHDRSZ);
+	size = asgeojson_triangle_buf(tri, srs, output->data, bbox, precision);
+	LWSIZE_SET(output->size, size + LWVARHDRSZ);
+	return output;
+}
 
 /**
  * Line Geometry
  */
 
 static size_t
-asgeojson_line_size(const LWLINE *line, char *srs, GBOX *bbox, int precision)
+asgeojson_line_size(const LWLINE *line, const char *srs, GBOX *bbox, int precision)
 {
 	int size;
 
@@ -230,7 +275,7 @@ asgeojson_line_size(const LWLINE *line, char *srs, GBOX *bbox, int precision)
 }
 
 static size_t
-asgeojson_line_buf(const LWLINE *line, char *srs, char *output, GBOX *bbox, int precision)
+asgeojson_line_buf(const LWLINE *line, const char *srs, char *output, GBOX *bbox, int precision)
 {
 	char *ptr=output;
 
@@ -244,16 +289,13 @@ asgeojson_line_buf(const LWLINE *line, char *srs, char *output, GBOX *bbox, int
 	return (ptr-output);
 }
 
-static char *
-asgeojson_line(const LWLINE *line, char *srs, GBOX *bbox, int precision)
+static lwvarlena_t *
+asgeojson_line(const LWLINE *line, const char *srs, GBOX *bbox, int precision)
 {
-	char *output;
-	int size;
-
-	size = asgeojson_line_size(line, srs, bbox, precision);
-	output = lwalloc(size);
-	asgeojson_line_buf(line, srs, output, bbox, precision);
-
+	uint32_t size = asgeojson_line_size(line, srs, bbox, precision);
+	lwvarlena_t *output = (lwvarlena_t *)lwalloc(size + LWVARHDRSZ);
+	size = asgeojson_line_buf(line, srs, output->data, bbox, precision);
+	LWSIZE_SET(output->size, size + LWVARHDRSZ);
 	return output;
 }
 
@@ -264,7 +306,7 @@ asgeojson_line(const LWLINE *line, char *srs, GBOX *bbox, int precision)
  */
 
 static size_t
-asgeojson_poly_size(const LWPOLY *poly, char *srs, GBOX *bbox, int precision)
+asgeojson_poly_size(const LWPOLY *poly, const char *srs, GBOX *bbox, int precision)
 {
 	size_t size;
 	uint32_t i;
@@ -285,9 +327,10 @@ asgeojson_poly_size(const LWPOLY *poly, char *srs, GBOX *bbox, int precision)
 }
 
 static size_t
-asgeojson_poly_buf(const LWPOLY *poly, char *srs, char *output, GBOX *bbox, int precision)
+asgeojson_poly_buf(const LWPOLY *poly, const char *srs, char *output, GBOX *bbox, int precision)
 {
 	uint32_t i;
+
 	char *ptr=output;
 
 	ptr += sprintf(ptr, "{\"type\":\"Polygon\",");
@@ -306,16 +349,13 @@ asgeojson_poly_buf(const LWPOLY *poly, char *srs, char *output, GBOX *bbox, int
 	return (ptr-output);
 }
 
-static char *
-asgeojson_poly(const LWPOLY *poly, char *srs, GBOX *bbox, int precision)
+static lwvarlena_t *
+asgeojson_poly(const LWPOLY *poly, const char *srs, GBOX *bbox, int precision)
 {
-	char *output;
-	int size;
-
-	size = asgeojson_poly_size(poly, srs, bbox, precision);
-	output = lwalloc(size);
-	asgeojson_poly_buf(poly, srs, output, bbox, precision);
-
+	uint32_t size = asgeojson_poly_size(poly, srs, bbox, precision);
+	lwvarlena_t *output = (lwvarlena_t *)lwalloc(size + LWVARHDRSZ);
+	size = asgeojson_poly_buf(poly, srs, output->data, bbox, precision);
+	LWSIZE_SET(output->size, size + LWVARHDRSZ);
 	return output;
 }
 
@@ -326,18 +366,21 @@ asgeojson_poly(const LWPOLY *poly, char *srs, GBOX *bbox, int precision)
  */
 
 static size_t
-asgeojson_multipoint_size(const LWMPOINT *mpoint, char *srs, GBOX *bbox, int precision)
+asgeojson_multipoint_size(const LWMPOINT *mpoint, const char *srs, GBOX *bbox, int precision)
 {
 	LWPOINT * point;
 	int size;
-	uint32_t i;
+	uint32_t i, ngeoms = mpoint->ngeoms;
 
 	size = sizeof("{'type':'MultiPoint',");
 	if (srs) size += asgeojson_srs_size(srs);
 	if (bbox) size += asgeojson_bbox_size(FLAGS_GET_Z(mpoint->flags), precision);
 	size += sizeof("'coordinates':[]}");
 
-	for (i=0; i<mpoint->ngeoms; i++)
+	if (lwgeom_is_empty((LWGEOM*)mpoint))
+		ngeoms = 0;
+
+	for (i=0; i<ngeoms; i++)
 	{
 		point = mpoint->geoms[i];
 		size += pointArray_geojson_size(point->point, precision);
@@ -348,10 +391,10 @@ asgeojson_multipoint_size(const LWMPOINT *mpoint, char *srs, GBOX *bbox, int pre
 }
 
 static size_t
-asgeojson_multipoint_buf(const LWMPOINT *mpoint, char *srs, char *output, GBOX *bbox, int precision)
+asgeojson_multipoint_buf(const LWMPOINT *mpoint, const char *srs, char *output, GBOX *bbox, int precision)
 {
 	LWPOINT *point;
-	uint32_t i;
+	uint32_t i, ngeoms = mpoint->ngeoms;
 	char *ptr=output;
 
 	ptr += sprintf(ptr, "{\"type\":\"MultiPoint\",");
@@ -359,7 +402,10 @@ asgeojson_multipoint_buf(const LWMPOINT *mpoint, char *srs, char *output, GBOX *
 	if (bbox) ptr += asgeojson_bbox_buf(ptr, bbox, FLAGS_GET_Z(mpoint->flags), precision);
 	ptr += sprintf(ptr, "\"coordinates\":[");
 
-	for (i=0; i<mpoint->ngeoms; i++)
+	if (lwgeom_is_empty((LWGEOM*)mpoint))
+		ngeoms = 0;
+
+	for (i=0; i<ngeoms; i++)
 	{
 		if (i) ptr += sprintf(ptr, ",");
 		point = mpoint->geoms[i];
@@ -370,16 +416,13 @@ asgeojson_multipoint_buf(const LWMPOINT *mpoint, char *srs, char *output, GBOX *
 	return (ptr - output);
 }
 
-static char *
-asgeojson_multipoint(const LWMPOINT *mpoint, char *srs, GBOX *bbox, int precision)
+static lwvarlena_t *
+asgeojson_multipoint(const LWMPOINT *mpoint, const char *srs, GBOX *bbox, int precision)
 {
-	char *output;
-	int size;
-
-	size = asgeojson_multipoint_size(mpoint, srs, bbox, precision);
-	output = lwalloc(size);
-	asgeojson_multipoint_buf(mpoint, srs, output, bbox, precision);
-
+	uint32_t size = asgeojson_multipoint_size(mpoint, srs, bbox, precision);
+	lwvarlena_t *output = (lwvarlena_t *)lwalloc(size + LWVARHDRSZ);
+	size = asgeojson_multipoint_buf(mpoint, srs, output->data, bbox, precision);
+	LWSIZE_SET(output->size, size + LWVARHDRSZ);
 	return output;
 }
 
@@ -390,18 +433,21 @@ asgeojson_multipoint(const LWMPOINT *mpoint, char *srs, GBOX *bbox, int precisio
  */
 
 static size_t
-asgeojson_multiline_size(const LWMLINE *mline, char *srs, GBOX *bbox, int precision)
+asgeojson_multiline_size(const LWMLINE *mline, const char *srs, GBOX *bbox, int precision)
 {
 	LWLINE * line;
 	int size;
-	uint32_t i;
+	uint32_t i, ngeoms = mline->ngeoms;
 
 	size = sizeof("{'type':'MultiLineString',");
 	if (srs) size += asgeojson_srs_size(srs);
 	if (bbox) size += asgeojson_bbox_size(FLAGS_GET_Z(mline->flags), precision);
 	size += sizeof("'coordinates':[]}");
 
-	for (i=0 ; i<mline->ngeoms; i++)
+	if (lwgeom_is_empty((LWGEOM*)mline))
+		ngeoms = 0;
+
+	for (i=0 ; i<ngeoms; i++)
 	{
 		line = mline->geoms[i];
 		size += pointArray_geojson_size(line->points, precision);
@@ -413,10 +459,10 @@ asgeojson_multiline_size(const LWMLINE *mline, char *srs, GBOX *bbox, int precis
 }
 
 static size_t
-asgeojson_multiline_buf(const LWMLINE *mline, char *srs, char *output, GBOX *bbox, int precision)
+asgeojson_multiline_buf(const LWMLINE *mline, const char *srs, char *output, GBOX *bbox, int precision)
 {
 	LWLINE *line;
-	uint32_t i;
+	uint32_t i, ngeoms = mline->ngeoms;
 	char *ptr=output;
 
 	ptr += sprintf(ptr, "{\"type\":\"MultiLineString\",");
@@ -424,7 +470,10 @@ asgeojson_multiline_buf(const LWMLINE *mline, char *srs, char *output, GBOX *bbo
 	if (bbox) ptr += asgeojson_bbox_buf(ptr, bbox, FLAGS_GET_Z(mline->flags), precision);
 	ptr += sprintf(ptr, "\"coordinates\":[");
 
-	for (i=0; i<mline->ngeoms; i++)
+	if (lwgeom_is_empty((LWGEOM*)mline))
+		ngeoms = 0;
+
+	for (i=0; i<ngeoms; i++)
 	{
 		if (i) ptr += sprintf(ptr, ",");
 		ptr += sprintf(ptr, "[");
@@ -438,16 +487,13 @@ asgeojson_multiline_buf(const LWMLINE *mline, char *srs, char *output, GBOX *bbo
 	return (ptr - output);
 }
 
-static char *
-asgeojson_multiline(const LWMLINE *mline, char *srs, GBOX *bbox, int precision)
+static lwvarlena_t *
+asgeojson_multiline(const LWMLINE *mline, const char *srs, GBOX *bbox, int precision)
 {
-	char *output;
-	int size;
-
-	size = asgeojson_multiline_size(mline, srs, bbox, precision);
-	output = lwalloc(size);
-	asgeojson_multiline_buf(mline, srs, output, bbox, precision);
-
+	uint32_t size = asgeojson_multiline_size(mline, srs, bbox, precision);
+	lwvarlena_t *output = (lwvarlena_t *)lwalloc(size + LWVARHDRSZ);
+	size = asgeojson_multiline_buf(mline, srs, output->data, bbox, precision);
+	LWSIZE_SET(output->size, size + LWVARHDRSZ);
 	return output;
 }
 
@@ -458,18 +504,21 @@ asgeojson_multiline(const LWMLINE *mline, char *srs, GBOX *bbox, int precision)
  */
 
 static size_t
-asgeojson_multipolygon_size(const LWMPOLY *mpoly, char *srs, GBOX *bbox, int precision)
+asgeojson_multipolygon_size(const LWMPOLY *mpoly, const char *srs, GBOX *bbox, int precision)
 {
 	LWPOLY *poly;
 	int size;
-	uint32_t i, j;
+	uint32_t i, j, ngeoms = mpoly->ngeoms;
 
 	size = sizeof("{'type':'MultiPolygon',");
 	if (srs) size += asgeojson_srs_size(srs);
 	if (bbox) size += asgeojson_bbox_size(FLAGS_GET_Z(mpoly->flags), precision);
 	size += sizeof("'coordinates':[]}");
 
-	for (i=0; i < mpoly->ngeoms; i++)
+	if (lwgeom_is_empty((LWGEOM*)mpoly))
+		ngeoms = 0;
+
+	for (i=0; i < ngeoms; i++)
 	{
 		poly = mpoly->geoms[i];
 		for (j=0 ; j <poly->nrings ; j++)
@@ -486,17 +535,21 @@ asgeojson_multipolygon_size(const LWMPOLY *mpoly, char *srs, GBOX *bbox, int pre
 }
 
 static size_t
-asgeojson_multipolygon_buf(const LWMPOLY *mpoly, char *srs, char *output, GBOX *bbox, int precision)
+asgeojson_multipolygon_buf(const LWMPOLY *mpoly, const char *srs, char *output, GBOX *bbox, int precision)
 {
 	LWPOLY *poly;
-	uint32_t i, j;
+	uint32_t i, j, ngeoms = mpoly->ngeoms;
 	char *ptr=output;
 
 	ptr += sprintf(ptr, "{\"type\":\"MultiPolygon\",");
 	if (srs) ptr += asgeojson_srs_buf(ptr, srs);
 	if (bbox) ptr += asgeojson_bbox_buf(ptr, bbox, FLAGS_GET_Z(mpoly->flags), precision);
 	ptr += sprintf(ptr, "\"coordinates\":[");
-	for (i=0; i<mpoly->ngeoms; i++)
+
+	if (lwgeom_is_empty((LWGEOM*)mpoly))
+		ngeoms = 0;
+
+	for (i=0; i < ngeoms; i++)
 	{
 		if (i) ptr += sprintf(ptr, ",");
 		ptr += sprintf(ptr, "[");
@@ -515,16 +568,13 @@ asgeojson_multipolygon_buf(const LWMPOLY *mpoly, char *srs, char *output, GBOX *
 	return (ptr - output);
 }
 
-static char *
-asgeojson_multipolygon(const LWMPOLY *mpoly, char *srs, GBOX *bbox, int precision)
+static lwvarlena_t *
+asgeojson_multipolygon(const LWMPOLY *mpoly, const char *srs, GBOX *bbox, int precision)
 {
-	char *output;
-	int size;
-
-	size = asgeojson_multipolygon_size(mpoly, srs, bbox, precision);
-	output = lwalloc(size);
-	asgeojson_multipolygon_buf(mpoly, srs, output, bbox, precision);
-
+	uint32_t size = asgeojson_multipolygon_size(mpoly, srs, bbox, precision);
+	lwvarlena_t *output = (lwvarlena_t *)lwalloc(size + LWVARHDRSZ);
+	size = asgeojson_multipolygon_buf(mpoly, srs, output->data, bbox, precision);
+	LWSIZE_SET(output->size, size + LWVARHDRSZ);
 	return output;
 }
 
@@ -535,9 +585,9 @@ asgeojson_multipolygon(const LWMPOLY *mpoly, char *srs, GBOX *bbox, int precisio
  */
 
 static size_t
-asgeojson_collection_size(const LWCOLLECTION *col, char *srs, GBOX *bbox, int precision)
+asgeojson_collection_size(const LWCOLLECTION *col, const char *srs, GBOX *bbox, int precision)
 {
-	uint32_t i;
+	uint32_t i, ngeoms = col->ngeoms;
 	size_t size;
 	LWGEOM *subgeom;
 
@@ -546,7 +596,10 @@ asgeojson_collection_size(const LWCOLLECTION *col, char *srs, GBOX *bbox, int pr
 	if (bbox) size += asgeojson_bbox_size(FLAGS_GET_Z(col->flags), precision);
 	size += sizeof("'geometries':");
 
-	for (i=0; i<col->ngeoms; i++)
+	if (lwgeom_is_empty((LWGEOM*)col))
+		ngeoms = 0;
+
+	for (i=0; i<ngeoms; i++)
 	{
 		subgeom = col->geoms[i];
 		size += asgeojson_geom_size(subgeom, NULL, precision);
@@ -558,9 +611,9 @@ asgeojson_collection_size(const LWCOLLECTION *col, char *srs, GBOX *bbox, int pr
 }
 
 static size_t
-asgeojson_collection_buf(const LWCOLLECTION *col, char *srs, char *output, GBOX *bbox, int precision)
+asgeojson_collection_buf(const LWCOLLECTION *col, const char *srs, char *output, GBOX *bbox, int precision)
 {
-	uint32_t i;
+	uint32_t i, ngeoms = col->ngeoms;
 	char *ptr=output;
 	LWGEOM *subgeom;
 
@@ -569,7 +622,10 @@ asgeojson_collection_buf(const LWCOLLECTION *col, char *srs, char *output, GBOX
 	if (col->ngeoms && bbox) ptr += asgeojson_bbox_buf(ptr, bbox, FLAGS_GET_Z(col->flags), precision);
 	ptr += sprintf(ptr, "\"geometries\":[");
 
-	for (i=0; i<col->ngeoms; i++)
+	if (lwgeom_is_empty((LWGEOM*)col))
+		ngeoms = 0;
+
+	for (i=0; i<ngeoms; i++)
 	{
 		if (i) ptr += sprintf(ptr, ",");
 		subgeom = col->geoms[i];
@@ -581,16 +637,13 @@ asgeojson_collection_buf(const LWCOLLECTION *col, char *srs, char *output, GBOX
 	return (ptr - output);
 }
 
-static char *
-asgeojson_collection(const LWCOLLECTION *col, char *srs, GBOX *bbox, int precision)
+static lwvarlena_t *
+asgeojson_collection(const LWCOLLECTION *col, const char *srs, GBOX *bbox, int precision)
 {
-	char *output;
-	int size;
-
-	size = asgeojson_collection_size(col, srs, bbox, precision);
-	output = lwalloc(size);
-	asgeojson_collection_buf(col, srs, output, bbox, precision);
-
+	uint32_t size = asgeojson_collection_size(col, srs, bbox, precision);
+	lwvarlena_t *output = (lwvarlena_t *)lwalloc(size + LWVARHDRSZ);
+	size = asgeojson_collection_buf(col, srs, output->data, bbox, precision);
+	LWSIZE_SET(output->size, size + LWVARHDRSZ);
 	return output;
 }
 
@@ -599,40 +652,26 @@ asgeojson_collection(const LWCOLLECTION *col, char *srs, GBOX *bbox, int precisi
 static size_t
 asgeojson_geom_size(const LWGEOM *geom, GBOX *bbox, int precision)
 {
-	int type = geom->type;
-	size_t size = 0;
-
-	switch (type)
+	switch (geom->type)
 	{
 	case POINTTYPE:
-		size = asgeojson_point_size((LWPOINT*)geom, NULL, bbox, precision);
-		break;
-
+		return asgeojson_point_size((LWPOINT *)geom, NULL, bbox, precision);
 	case LINETYPE:
-		size = asgeojson_line_size((LWLINE*)geom, NULL, bbox, precision);
-		break;
-
+		return asgeojson_line_size((LWLINE *)geom, NULL, bbox, precision);
+	case TRIANGLETYPE:
+		return asgeojson_triangle_size((LWTRIANGLE *)geom, NULL, bbox, precision);
 	case POLYGONTYPE:
-		size = asgeojson_poly_size((LWPOLY*)geom, NULL, bbox, precision);
-		break;
-
+		return asgeojson_poly_size((LWPOLY *)geom, NULL, bbox, precision);
 	case MULTIPOINTTYPE:
-		size = asgeojson_multipoint_size((LWMPOINT*)geom, NULL, bbox, precision);
-		break;
-
+		return asgeojson_multipoint_size((LWMPOINT *)geom, NULL, bbox, precision);
 	case MULTILINETYPE:
-		size = asgeojson_multiline_size((LWMLINE*)geom, NULL, bbox, precision);
-		break;
-
+		return asgeojson_multiline_size((LWMLINE *)geom, NULL, bbox, precision);
 	case MULTIPOLYGONTYPE:
-		size = asgeojson_multipolygon_size((LWMPOLY*)geom, NULL, bbox, precision);
-		break;
-
+		return asgeojson_multipolygon_size((LWMPOLY *)geom, NULL, bbox, precision);
 	default:
 		lwerror("GeoJson: geometry not supported.");
+		return 0;
 	}
-
-	return size;
 }
 
 
@@ -656,6 +695,10 @@ asgeojson_geom_buf(const LWGEOM *geom, char *output, GBOX *bbox, int precision)
 		ptr += asgeojson_poly_buf((LWPOLY*)geom, NULL, ptr, bbox, precision);
 		break;
 
+	case TRIANGLETYPE:
+		ptr += asgeojson_triangle_buf((LWTRIANGLE *)geom, NULL, ptr, bbox, precision);
+		break;
+
 	case MULTIPOINTTYPE:
 		ptr += asgeojson_multipoint_buf((LWMPOINT*)geom, NULL, ptr, bbox, precision);
 		break;
@@ -679,50 +722,54 @@ asgeojson_geom_buf(const LWGEOM *geom, char *output, GBOX *bbox, int precision)
 static size_t
 pointArray_to_geojson(POINTARRAY *pa, char *output, int precision)
 {
-	uint32_t i;
-	char *ptr;
-	char x[OUT_DOUBLE_BUFFER_SIZE];
-	char y[OUT_DOUBLE_BUFFER_SIZE];
-	char z[OUT_DOUBLE_BUFFER_SIZE];
+	char *ptr = output;
 
-	assert ( precision <= OUT_MAX_DOUBLE_PRECISION );
-	ptr = output;
-
-	/* TODO: rewrite this loop to be simpler and possibly quicker */
 	if (!FLAGS_GET_Z(pa->flags))
 	{
-		for (i=0; i<pa->npoints; i++)
+		for (uint32_t i = 0; i < pa->npoints; i++)
 		{
-			const POINT2D *pt;
-			pt = getPoint2d_cp(pa, i);
+			if (i)
+			{
+				*ptr = ',';
+				ptr++;
+			}
+			const POINT2D *pt = getPoint2d_cp(pa, i);
 
-			lwprint_double(
-			    pt->x, precision, x, OUT_DOUBLE_BUFFER_SIZE);
-			lwprint_double(
-			    pt->y, precision, y, OUT_DOUBLE_BUFFER_SIZE);
-
-			if ( i ) ptr += sprintf(ptr, ",");
-			ptr += sprintf(ptr, "[%s,%s]", x, y);
+			*ptr = '[';
+			ptr++;
+			ptr += lwprint_double(pt->x, precision, ptr);
+			*ptr = ',';
+			ptr++;
+			ptr += lwprint_double(pt->y, precision, ptr);
+			*ptr = ']';
+			ptr++;
 		}
 	}
 	else
 	{
-		for (i=0; i<pa->npoints; i++)
+		for (uint32_t i = 0; i < pa->npoints; i++)
 		{
-			const POINT3DZ *pt;
-			pt = getPoint3dz_cp(pa, i);
+			if (i)
+			{
+				*ptr = ',';
+				ptr++;
+			}
 
-			lwprint_double(
-			    pt->x, precision, x, OUT_DOUBLE_BUFFER_SIZE);
-			lwprint_double(
-			    pt->y, precision, y, OUT_DOUBLE_BUFFER_SIZE);
-			lwprint_double(
-			    pt->z, precision, z, OUT_DOUBLE_BUFFER_SIZE);
-
-			if ( i ) ptr += sprintf(ptr, ",");
-			ptr += sprintf(ptr, "[%s,%s,%s]", x, y, z);
+			const POINT3D *pt = getPoint3d_cp(pa, i);
+			*ptr = '[';
+			ptr++;
+			ptr += lwprint_double(pt->x, precision, ptr);
+			*ptr = ',';
+			ptr++;
+			ptr += lwprint_double(pt->y, precision, ptr);
+			*ptr = ',';
+			ptr++;
+			ptr += lwprint_double(pt->z, precision, ptr);
+			*ptr = ']';
+			ptr++;
 		}
 	}
+	*ptr = '\0';
 
 	return (ptr-output);
 }
@@ -733,11 +780,8 @@ pointArray_to_geojson(POINTARRAY *pa, char *output, int precision)
 static size_t
 pointArray_geojson_size(POINTARRAY *pa, int precision)
 {
-	assert ( precision <= OUT_MAX_DOUBLE_PRECISION );
 	if (FLAGS_NDIMS(pa->flags) == 2)
-		return (OUT_MAX_DIGS_DOUBLE + precision + sizeof(","))
-		       * 2 * pa->npoints + sizeof(",[]");
+		return (OUT_MAX_BYTES_DOUBLE + precision + sizeof(",")) * 2 * pa->npoints + sizeof(",[]");
 
-	return (OUT_MAX_DIGS_DOUBLE + precision + sizeof(",,"))
-	       * 3 * pa->npoints + sizeof(",[]");
+	return (OUT_MAX_BYTES_DOUBLE + precision + sizeof(",,")) * 3 * pa->npoints + sizeof(",[]");
 }
