@@ -126,10 +126,10 @@ lwpoint_get_m(const LWPOINT *point)
  * use SRID=SRID_UNKNOWN for unknown SRID (will have 8bit type's S = 0)
  */
 LWPOINT *
-lwpoint_construct(int srid, GBOX *bbox, POINTARRAY *point)
+lwpoint_construct(int32_t srid, GBOX *bbox, POINTARRAY *point)
 {
 	LWPOINT *result;
-	uint8_t flags = 0;
+	lwflags_t flags = 0;
 
 	if (point == NULL)
 		return NULL; /* error */
@@ -148,11 +148,11 @@ lwpoint_construct(int srid, GBOX *bbox, POINTARRAY *point)
 }
 
 LWPOINT *
-lwpoint_construct_empty(int srid, char hasz, char hasm)
+lwpoint_construct_empty(int32_t srid, char hasz, char hasm)
 {
 	LWPOINT *result = lwalloc(sizeof(LWPOINT));
 	result->type = POINTTYPE;
-	result->flags = gflags(hasz, hasm, 0);
+	result->flags = lwflags(hasz, hasm, 0);
 	result->srid = srid;
 	result->point = ptarray_construct(hasz, hasm, 0);
 	result->bbox = NULL;
@@ -160,7 +160,7 @@ lwpoint_construct_empty(int srid, char hasz, char hasm)
 }
 
 LWPOINT *
-lwpoint_make2d(int srid, double x, double y)
+lwpoint_make2d(int32_t srid, double x, double y)
 {
 	POINT4D p = {x, y, 0.0, 0.0};
 	POINTARRAY *pa = ptarray_construct_empty(0, 0, 1);
@@ -170,7 +170,7 @@ lwpoint_make2d(int srid, double x, double y)
 }
 
 LWPOINT *
-lwpoint_make3dz(int srid, double x, double y, double z)
+lwpoint_make3dz(int32_t srid, double x, double y, double z)
 {
 	POINT4D p = {x, y, z, 0.0};
 	POINTARRAY *pa = ptarray_construct_empty(1, 0, 1);
@@ -181,7 +181,7 @@ lwpoint_make3dz(int srid, double x, double y, double z)
 }
 
 LWPOINT *
-lwpoint_make3dm(int srid, double x, double y, double m)
+lwpoint_make3dm(int32_t srid, double x, double y, double m)
 {
 	POINT4D p = {x, y, 0.0, m};
 	POINTARRAY *pa = ptarray_construct_empty(0, 1, 1);
@@ -192,7 +192,7 @@ lwpoint_make3dm(int srid, double x, double y, double m)
 }
 
 LWPOINT *
-lwpoint_make4d(int srid, double x, double y, double z, double m)
+lwpoint_make4d(int32_t srid, double x, double y, double z, double m)
 {
 	POINT4D p = {x, y, z, m};
 	POINTARRAY *pa = ptarray_construct_empty(1, 1, 1);
@@ -203,7 +203,7 @@ lwpoint_make4d(int srid, double x, double y, double z, double m)
 }
 
 LWPOINT *
-lwpoint_make(int srid, int hasz, int hasm, const POINT4D *p)
+lwpoint_make(int32_t srid, int hasz, int hasm, const POINT4D *p)
 {
 	POINTARRAY *pa = ptarray_construct_empty(hasz, hasm, 1);
 	ptarray_append_point(pa, p, LW_TRUE);
@@ -268,7 +268,7 @@ lwpoint_same(const LWPOINT *p1, const LWPOINT *p2)
 
 
 LWPOINT*
-lwpoint_force_dims(const LWPOINT *point, int hasz, int hasm)
+lwpoint_force_dims(const LWPOINT *point, int hasz, int hasm, double zval, double mval)
 {
 	POINTARRAY *pdims = NULL;
 	LWPOINT *pointout;
@@ -281,25 +281,11 @@ lwpoint_force_dims(const LWPOINT *point, int hasz, int hasm)
 	else
 	{
 		/* Always we duplicate the ptarray and return */
-		pdims = ptarray_force_dims(point->point, hasz, hasm);
+		pdims = ptarray_force_dims(point->point, hasz, hasm, zval, mval);
 		pointout = lwpoint_construct(point->srid, NULL, pdims);
 	}
 	pointout->type = point->type;
 	return pointout;
 }
 
-int lwpoint_is_empty(const LWPOINT *point)
-{
-	if ( ! point->point || point->point->npoints < 1 )
-		return LW_TRUE;
-	return LW_FALSE;
-}
-
-
-LWPOINT *
-lwpoint_grid(const LWPOINT *point, const gridspec *grid)
-{
-	POINTARRAY *opa = ptarray_grid(point->point, grid);
-	return lwpoint_construct(point->srid, NULL, opa);
-}
 
