@@ -521,6 +521,8 @@ Datum geometry_distance_spheroid(PG_FUNCTION_ARGS)
 	/* We are going to be calculating geodetic distances */
 	lwgeom_set_geodetic(lwgeom1, LW_TRUE);
 	lwgeom_set_geodetic(lwgeom2, LW_TRUE);
+	lwgeom_refresh_bbox(lwgeom1);
+	lwgeom_refresh_bbox(lwgeom2);
 
 	distance = lwgeom_distance_spheroid(lwgeom1, lwgeom2, sphere, 0.0);
 
@@ -531,17 +533,36 @@ Datum geometry_distance_spheroid(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_distance_ellipsoid);
 Datum LWGEOM_distance_ellipsoid(PG_FUNCTION_ARGS)
 {
+	SPHEROID s;
+
+	/* No spheroid provided */
+	if (PG_NARGS() == 2) {
+		/* Init to WGS84 */
+		spheroid_init(&s, 6378137.0, 6356752.314245179498);
+		PG_RETURN_DATUM(DirectFunctionCall4(geometry_distance_spheroid,
+			PG_GETARG_DATUM(0),
+			PG_GETARG_DATUM(1),
+			PointerGetDatum(&s),
+			BoolGetDatum(true)));
+	}
+
 	PG_RETURN_DATUM(DirectFunctionCall4(geometry_distance_spheroid,
-	                                    PG_GETARG_DATUM(0), PG_GETARG_DATUM(1), PG_GETARG_DATUM(2), BoolGetDatum(true)));
+		PG_GETARG_DATUM(0),
+		PG_GETARG_DATUM(1),
+		PG_GETARG_DATUM(2),
+		BoolGetDatum(true)));
 }
 
 PG_FUNCTION_INFO_V1(LWGEOM_distance_sphere);
 Datum LWGEOM_distance_sphere(PG_FUNCTION_ARGS)
 {
 	SPHEROID s;
-
 	/* Init to WGS84 */
 	spheroid_init(&s, 6378137.0, 6356752.314245179498);
+
+	if (PG_NARGS() == 3) {
+		s.radius = PG_GETARG_FLOAT8(2);
+	}
 	s.a = s.b = s.radius;
 
 	PG_RETURN_DATUM(DirectFunctionCall4(geometry_distance_spheroid,
