@@ -139,6 +139,8 @@
 
 #include "rtpostgis.h"
 #include "rtpg_internal.h"
+#include "stringlist.h"
+#include "optionlist.h"
 
 #ifndef __GNUC__
 # define __attribute__ (x)
@@ -239,14 +241,220 @@ rt_pg_debug(const char *fmt, va_list ap)
     ereport(DEBUG1, (errmsg_internal("%s", msg)));
 }
 
+static char *
+rt_pg_options(const char* varname)
+{
+	char optname[128];
+	char *optvalue;
+	snprintf(optname, 128, "postgis.%s", varname);
+	/* GetConfigOptionByName(name, found_name, missing_ok) */
+	optvalue = GetConfigOptionByName(optname, NULL, true);
+	if (optvalue && strlen(optvalue) == 0)
+		return NULL;
+	else
+		return optvalue;
+}
+
+/* ---------------------------------------------------------------- */
+/*  GDAL allowed config options for VSI filesystems */
+/* ---------------------------------------------------------------- */
+
+stringlist_t *vsi_option_stringlist = NULL;
+
+
+#if POSTGIS_GDAL_VERSION < 23
+
+/*
+* For older versions of GDAL we  have extracted the list of options
+* that were available at the 2.2 release and use that
+* as our set of allowed VSI network file options.
+*/
+static void
+rt_pg_vsi_load_all_options(void)
+{
+	const char * gdaloption;
+	const char * const gdaloptions[] = {
+		"aws_access_key_id",
+		"aws_https",
+		"aws_max_keys",
+		"aws_s3_endpoint",
+		"aws_region",
+		"aws_request_payer",
+		"aws_secret_access_key",
+		"aws_session_token",
+		"aws_timestamp",
+		"aws_virtual_hosting",
+		"cpl_gs_timestamp",
+		"cpl_gs_endpoint",
+		"gs_secret_access_key",
+		"gs_access_key_id",
+		"goa2_client_id",
+		"goa2_client_secret",
+		"cpl_curl_enable_vsimem",
+		"cpl_curl_gzip",
+		"cpl_curl_verbose",
+		"gdal_http_auth",
+		"gdal_http_connecttimeout",
+		"gdal_http_cookie",
+		"gdal_http_header_file",
+		"gdal_http_low_speed_time",
+		"gdal_http_low_speed_limit",
+		"gdal_http_max_retry",
+		"gdal_http_netrc",
+		"gdal_http_proxy",
+		"gdal_http_proxyuserpwd",
+		"gdal_http_retry_delay",
+		"gdal_http_userpwd",
+		"gdal_http_timeout",
+		"gdal_http_unsafessl",
+		"gdal_http_useragent",
+		"gdal_disable_readdir_on_open",
+		"gdal_proxy_auth",
+		"curl_ca_bundle",
+		"ssl_cert_file",
+		"vsi_cache_size",
+		"cpl_vsil_curl_use_head",
+		"cpl_vsil_curl_use_s3_redirect",
+		"cpl_vsil_curl_max_ranges",
+		"cpl_vsil_curl_use_cache",
+		"cpl_vsil_curl_allowed_filename",
+		"cpl_vsil_curl_allowed_extensions",
+		"cpl_vsil_curl_slow_get_size",
+		"vsi_cache",
+		"vsis3_chunk_size",
+		NULL
+	};
+	const char * const * gdaloptionsptr = gdaloptions;
+
+	vsi_option_stringlist = stringlist_create();
+	while((gdaloption = *gdaloptionsptr++))
+	{
+		stringlist_add_string_nosort(vsi_option_stringlist, gdaloption);
+	}
+	stringlist_sort(vsi_option_stringlist);
+}
+
+#else /* POSTGIS_GDAL_VERSION < 23 */
+
+/*
+* For newer versions of GDAL the VSIGetFileSystemOptions() call returns
+* all the allowed options for each VSI network file type, and we just have
+* to keep the list of VSI types statically in rt_pg_vsi_load_all_options().
+*/
+static void
+rt_pg_vsi_load_options(const char* vsiname, stringlist_t *s)
+{
+	CPLXMLNode *root, *optNode;
+	const char *xml = VSIGetFileSystemOptions(vsiname);
+	if (!xml) return;
+
+	root = CPLParseXMLString(xml);
+	if (!root) {
+		elog(ERROR, "%s: Unable to read options for VSI %s", __func__, vsiname);
+		return;
+	}
+	optNode = CPLSearchXMLNode(root, "Option");
+	if (!optNode) {
+		CPLDestroyXMLNode(root);
+		elog(ERROR, "%s: Unable to find <Option> in VSI XML %s", __func__, vsiname);
+		return;
+	}
+	while(optNode)
+	{
+		const char *option = CPLGetXMLValue(optNode, "name", NULL);
+		if (option) {
+			char *optionstr = pstrdup(option);
+			char *ptr = optionstr;
+			/* The options parser used in rt_util_gdal_open()
+			   lowercases keys, so we'll lower case our list
+			   of options before storing them in the stringlist. */
+			while (*ptr) {
+				*ptr = tolower(*ptr);
+				ptr++;
+			}
+			elog(DEBUG4, "GDAL %s option: %s", vsiname, optionstr);
+			stringlist_add_string_nosort(s, optionstr);
+		}
+		optNode = optNode->psNext;
+	}
+	CPLDestroyXMLNode(root);
+}
+
+static void
+rt_pg_vsi_load_all_options(void)
+{
+	const char * vsiname;
+	const char * const vsilist[] = {
+		"/vsicurl/",
+		"/vsis3/",
+		"/vsigs/",
+		"/vsiaz/",
+		"/vsioss/",
+		"/vsihdfs/",
+		"/vsiwebhdfs/",
+		"/vsiswift/",
+		"/vsiadls/",
+		NULL
+	};
+	const char * const * vsilistptr = vsilist;
+
+	vsi_option_stringlist = stringlist_create();
+	while((vsiname = *vsilistptr++))
+	{
+		rt_pg_vsi_load_options(vsiname, vsi_option_stringlist);
+	}
+	stringlist_sort(vsi_option_stringlist);
+}
+
+#endif /* POSTGIS_GDAL_VERSION < 23 */
+
+
+static bool
+rt_pg_vsi_check_options(char **newval, void **extra, GucSource source)
+{
+	size_t olist_sz, i;
+	char *olist[OPTION_LIST_SIZE];
+	const char *found = NULL;
+	char *newoptions;
+
+	memset(olist, 0, sizeof(olist));
+	if (!newval || !*newval)
+		return false;
+	newoptions = pstrdup(*newval);
+
+	/* Cache the legal options if they aren't already loaded */
+	if (!vsi_option_stringlist)
+		rt_pg_vsi_load_all_options();
+
+	elog(DEBUG5, "%s: processing VSI options: %s", __func__, newoptions);
+	option_list_parse(newoptions, olist);
+	olist_sz = option_list_length(olist);
+	if (olist_sz % 2 != 0)
+		return false;
+
+	for (i = 0; i < olist_sz; i += 2)
+	{
+		found = stringlist_find(vsi_option_stringlist, olist[i]);
+		if (!found)
+		{
+			elog(WARNING, "'%s' is not a legal VSI network file option", olist[i]);
+			pfree(newoptions);
+			return false;
+		}
+	}
+	return true;
+}
+
 
 /* ---------------------------------------------------------------- */
 /*  PostGIS raster GUCs                                             */
 /* ---------------------------------------------------------------- */
 
 static char *gdal_datapath = NULL;
-extern char *gdal_enabled_drivers;
-extern bool enable_outdb_rasters;
+static char *gdal_vsi_options = NULL;
+
+static char *gdal_enabled_drivers = NULL;
+static bool enable_outdb_rasters = false;
 
 /* ---------------------------------------------------------------- */
 /*  Useful variables                                                */
@@ -430,6 +638,7 @@ rtpg_assignHookEnableOutDBRasters(bool enable, void *extra) {
 	/* do nothing for now */
 }
 
+
 /* Module load callback */
 void
 _PG_init(void) {
@@ -494,7 +703,9 @@ _PG_init(void) {
 	pg_install_lwgeom_handlers();
 
 	/* Install rtcore handlers */
-	rt_set_handlers(rt_pg_alloc, rt_pg_realloc, rt_pg_free, rt_pg_error, rt_pg_debug, rt_pg_notice);
+	rt_set_handlers_options(rt_pg_alloc, rt_pg_realloc, rt_pg_free,
+		rt_pg_error, rt_pg_debug, rt_pg_notice,
+		rt_pg_options);
 
 	/* Define custom GUC variables. */
 	if ( postgis_guc_find_option("postgis.gdal_datapath") )
@@ -560,12 +771,32 @@ _PG_init(void) {
 			boot_postgis_enable_outdb_rasters, /* bootValue */
 			PGC_SUSET, /* GucContext context */
 			0, /* int flags */
-			NULL, /* GucStringCheckHook check_hook */
+			NULL, /* GucBoolCheckHook check_hook */
 			rtpg_assignHookEnableOutDBRasters, /* GucBoolAssignHook assign_hook */
 			NULL  /* GucShowHook show_hook */
 		);
 	}
 
+	if ( postgis_guc_find_option("postgis.gdal_vsi_options") )
+	{
+		elog(WARNING, "'%s' is already set and cannot be changed until you reconnect", "postgis.gdal_vsi_options");
+	}
+	else
+	{
+		DefineCustomStringVariable(
+			"postgis.gdal_vsi_options", /* name */
+			"VSI config options", /* short_desc */
+			"Set the config options to be used when opening /vsi/ network files", /* long_desc */
+			&gdal_vsi_options, /* valueAddr */
+			"", /* bootValue */
+			PGC_USERSET, /* GucContext context */
+			0, /* int flags */
+			rt_pg_vsi_check_options, /* GucStringCheckHook check_hook */
+			NULL, /* GucStringAssignHook assign_hook */
+			NULL  /* GucShowHook show_hook */
+		);
+	}
+
 	/* Revert back to old context */
 	MemoryContextSwitchTo(old_context);
 }
